*$ CREATE SOURCE.FOR
*COPY SOURCE
*
*=== source ===========================================================*
*
      SUBROUTINE SOURCE ( NOMORE )

      INCLUDE '(DBLPRC)'
      INCLUDE '(DIMPAR)'
      INCLUDE '(IOUNIT)'
*
*----------------------------------------------------------------------*
*                                                                      *
*     Copyright (C) 1990-2006      by    Alfredo Ferrari & Paola Sala  *
*     All Rights Reserved.                                             *
*                                                                      *
*                                                                      *
*     New source for FLUKA9x-FLUKA200x:                                *
*                                                                      *
*     Created on 07 january 1990   by    Alfredo Ferrari & Paola Sala  *
*                                                   Infn - Milan       *
*                                                                      *
*     Last change on 03-mar-06     by    Alfredo Ferrari               *
*                                                                      *
*  This is just an example of a possible user written source routine.  *
*  note that the beam card still has some meaning - in the scoring the *
*  maximum momentum used in deciding the binning is taken from the     *
*  beam momentum.  Other beam card parameters are obsolete.            *
*                                                                      *
*----------------------------------------------------------------------*
*
      INCLUDE '(BEAMCM)'
      INCLUDE '(FHEAVY)'
      INCLUDE '(FLKSTK)'
      INCLUDE '(IOIOCM)'
      INCLUDE '(LTCLCM)'
      INCLUDE '(PAPROP)'
      INCLUDE '(SOURCM)'
      INCLUDE '(SUMCOU)'

      INCLUDE '(GENSTK)'
*
      LOGICAL LFIRST

      PARAMETER (NMAX=200000)
*
      SAVE LFIRST
      DATA LFIRST / .TRUE. /

      CHARACTER*250 LINE
      INTEGER    NIN
      DIMENSION  XI(NMAX), YI(NMAX),
     &           UI(NMAX), VI(NMAX), WI(NMAX), PI(NMAX)
      SAVE XI, YI, UI, VI, WI, PI, NIN

*======================================================================*
*                                                                      *
*                 BASIC VERSION                                        *
*                                                                      *
*======================================================================*
      NOMORE = 0
*  +-------------------------------------------------------------------*
*  |  First call initializations:
      IF ( LFIRST ) THEN
*  |  *** The following 3 cards are mandatory ***
         TKESUM = ZERZER
         LFIRST = .FALSE.
         LUSSRC = .TRUE.
*  |  *** User initialization ***

*  |  WHASOU(1) = UNIT number (File name is linked with OPEN)
*  |  WHASOU(2) = sigma(cm) of randomness in posision
*  |  WHASOU(3) = sigma(rad) of -//- in direction
         LUNRD = NINT(WHASOU(1))
         OPEN(LUNRD, FILE="../gamma_5GeV_NODRIFT.txt", STATUS="OLD")

         NIN = 0
 10      CONTINUE
            READ( LUNRD, '(A)', ERR=9999, END=20 ) LINE
            PRINT *,LINE
            READ (LINE,*,ERR=10) X, Y, PX, PY, PZ
            NIN = NIN + 1
            IF (NIN.GT.NMAX) CALL FLABRT('SOURCE','Increase NMAX')

            XI(NIN) = X
            YI(NIN) = Y
            PI(NIN) = SQRT (PX*PX + PY*PY + PZ*PZ)
            UI(NIN) = PX / PI(NIN)
            VI(NIN) = PY / PI(NIN)
            WI(NIN) = PZ / PI(NIN)
            PI(NIN) = 1.0E-3*PI(NIN)

         GOTO 10
 20      CONTINUE
         IF (NIN.EQ.0) CALL FLABRT('SOURCE','Error reading file')
         WRITE (LUNOUT,*) NIN,' particles loaded'

      END IF
*  |
*  +-------------------------------------------------------------------*
*  Push one source particle to the stack. Note that you could as well
*  push many but this way we reserve a maximum amount of space in the
*  stack for the secondaries to be generated
* Npflka is the stack counter: of course any time source is called it
* must be =0
      NPFLKA = NPFLKA + 1
* Wt is the weight of the particle
      WTFLK  (NPFLKA) = ONEONE
      WEIPRI = WEIPRI + WTFLK (NPFLKA)
* Particle type (1=proton.....). Ijbeam is the type set by the BEAM
* card
*  +-------------------------------------------------------------------*
*  |  (Radioactive) isotope:
      IF ( IJBEAM .EQ. -2 .AND. LRDBEA ) THEN
         IARES  = IPROA
         IZRES  = IPROZ
         IISRES = IPROM
         CALL STISBM ( IARES, IZRES, IISRES )
         IJHION = IPROZ  * 1000 + IPROA
         IJHION = IJHION * 100 + KXHEAV
         IONID  = IJHION
         CALL DCDION ( IONID )
         CALL SETION ( IONID )
*  |
*  +-------------------------------------------------------------------*
*  |  Heavy ion:
      ELSE IF ( IJBEAM .EQ. -2 ) THEN
         IJHION = IPROZ  * 1000 + IPROA
         IJHION = IJHION * 100 + KXHEAV
         IONID  = IJHION
         CALL DCDION ( IONID )
         CALL SETION ( IONID )
         ILOFLK (NPFLKA) = IJHION
*  |  Flag this is prompt radiation
         LRADDC (NPFLKA) = .FALSE.
*  |
*  +-------------------------------------------------------------------*
*  |  Normal hadron:
      ELSE
         IONID = IJBEAM
         ILOFLK (NPFLKA) = IJBEAM
*  |  Flag this is prompt radiation
         LRADDC (NPFLKA) = .FALSE.
      END IF
*  |1.3718E-04
*  +-------------------------------------------------------------------*

*  +-------------------------------------------------------------------*
*  |  Check inelastic interactions
C      IJ  = 1
C      EKE = 0.160
C      POO = SQRT ( EKE * ( EKE + TWOTWO * AM (IONID) ) )
C      MAT = 6 ! Carbon
C      TX = ONEONE
C      TY = ZERZER
C      TZ = ZERZER
C      TXP = ZERZER
C      TYP = ZERZER
C      TZP = ZERZER
C      WEE = ONEONE
C      BAINE = ZERZER
C      DO I=1,100000
C          NP = 0
C          CALL EVENTX(IJ,POO,EKE,TX,TY,TZ,TXP,TYP,TZP,WEE,MAT,BAINE)
C          DO J=1,NP
C             IF (KPART(J) .EQ. 1) THEN
C                 WRITE (98,*) I, TKI(J)
C             END IF
C          END DO
C      END DO
C      STOP
*  |
*  +-------------------------------------------------------------------*

* From this point .....
*
*  +-------------------------------------------------------------------*
*  | Choose a random particle
      RNDSIG = FLRNDM (RNDSIG)
      N = NINT(NIN*RNDSIG)+1
      IF (N.GT.NIN) N=NIN
*  |
*  +-------------------------------------------------------------------*

* Particle generation (1 for primaries)
      LOFLK  (NPFLKA) = 1
* User dependent flag:
      LOUSE  (NPFLKA) = 0
* User dependent spare variables:
      DO 100 ISPR = 1, MKBMX1
         SPAREK (ISPR,NPFLKA) = ZERZER
 100  CONTINUE
* User dependent spare flags:
      DO 200 ISPR = 1, MKBMX2
         ISPARK (ISPR,NPFLKA) = 0
 200  CONTINUE
* Save the track number of the stack particle:
      ISPARK (MKBMX2,NPFLKA) = NPFLKA
      NPARMA = NPARMA + 1
      NUMPAR (NPFLKA) = NPARMA
      NEVENT (NPFLKA) = 0
      DFNEAR (NPFLKA) = +ZERZER
* ... to this point: don't change anything
* Particle age (s)
      AGESTK (NPFLKA) = +ZERZER
      AKNSHR (NPFLKA) = -TWOTWO
* Group number for "low" energy neutrons, set to 0 anyway
      IGROUP (NPFLKA) = 0
* Particle momentum
      PMOFLK (NPFLKA) = PI(NIN)
*      PMOFLK (NPFLKA) = SQRT ( TKEFLK (NPFLKA) * ( TKEFLK (NPFLKA)
*     &                       + TWOTWO * AM (IONID) ) )
* Kinetic energy of the particle (GeV)
      TKEFLK (NPFLKA) = SQRT ( PI(NIN)**2 + AM (IONID)**2 ) - AM (IONID)
*      TKEFLK (NPFLKA) = ERG(N)
* Cosines (tx,ty,tz)
      TXFLK  (NPFLKA) = UI(NIN)
      TYFLK  (NPFLKA) = VI(NIN)
      TZFLK  (NPFLKA) = WI(NIN)
* Polarization cosines:
      TXPOL  (NPFLKA) = -TWOTWO
      TYPOL  (NPFLKA) = +ZERZER
      TZPOL  (NPFLKA) = +ZERZER
* Particle coordinates
* with randomness in position
      XFLK   (NPFLKA) = XBEAM + XI(N)
      YFLK   (NPFLKA) = YBEAM + YI(N)
      ZFLK   (NPFLKA) = ZBEAM

*     WRITE(99,*) N, XI(N), XFLK(NPFLKA),
*    &               YI(N), YFLK(NPFLKA),
*    &               UI(N), TXFLK(NPFLKA),
*    &               VI(N), TYFLK(NPFLKA)
*  Calculate the total kinetic energy of the primaries: don't change
      IF ( ILOFLK (NPFLKA) .EQ. -2 .OR. ILOFLK (NPFLKA) .GT. 100000 )
     &   THEN
         TKESUM = TKESUM + TKEFLK (NPFLKA) * WTFLK (NPFLKA)
      ELSE IF ( ILOFLK (NPFLKA) .NE. 0 ) THEN
         TKESUM = TKESUM + ( TKEFLK (NPFLKA) + AMDISC (ILOFLK(NPFLKA)) )
     &          * WTFLK (NPFLKA)
      ELSE
         TKESUM = TKESUM + TKEFLK (NPFLKA) * WTFLK (NPFLKA)
      END IF
      RADDLY (NPFLKA) = ZERZER
*  Here we ask for the region number of the hitting point.
*     NREG (NPFLKA) = ...
*  The following line makes the starting region search much more
*  robust if particles are starting very close to a boundary:
      CALL GEOCRS ( TXFLK (NPFLKA), TYFLK (NPFLKA), TZFLK (NPFLKA) )
      CALL GEOREG ( XFLK  (NPFLKA), YFLK  (NPFLKA), ZFLK  (NPFLKA),
     &              NRGFLK(NPFLKA), IDISC )
*  Do not change these cards:
      CALL GEOHSM ( NHSPNT (NPFLKA), 1, -11, MLATTC )
      NLATTC (NPFLKA) = MLATTC
      CMPATH (NPFLKA) = ZERZER
      CALL SOEVSV
      RETURN
*=== End of subroutine Source =========================================*
 9999 CONTINUE
      CALL FLABRT('SOURCE','Error reading source file')
      END

